import asyncio
import aiomysql


loop = asyncio.get_event_loop()


@asyncio.coroutine
def test_example_transaction():
    conn = yield from aiomysql.connect(host='127.0.0.1', port=3306,
                                       user='root', password='',
                                       db='test_pymysql', autocommit=False,
                                       loop=loop)

    cursor = yield from conn.cursor()
    stmt_drop = "DROP TABLE IF EXISTS names"
    yield from cursor.execute(stmt_drop)
    yield from cursor.execute("""
        CREATE TABLE names (
        id TINYINT UNSIGNED NOT NULL AUTO_INCREMENT,
        name VARCHAR(30) DEFAULT '' NOT NULL,
        cnt TINYINT UNSIGNED DEFAULT 0,
        PRIMARY KEY (id))""")
    yield from conn.commit()

    # Insert 3 records
    names = (('Geert',), ('Jan',), ('Michel',))
    stmt_insert = "INSERT INTO names (name) VALUES (%s)"
    yield from cursor.executemany(stmt_insert, names)

    # Roll back!!!!
    yield from conn.rollback()

    # There should be no data!
    stmt_select = "SELECT id, name FROM names ORDER BY id"
    yield from cursor.execute(stmt_select)
    resp = yield from cursor.fetchall()
    # Check there is no data
    assert not resp

    # Do the insert again.
    yield from cursor.executemany(stmt_insert, names)

    # Data should be already there
    yield from cursor.execute(stmt_select)
    resp = yield from cursor.fetchall()
    print(resp)
    # Do a commit
    yield from conn.commit()

    yield from cursor.execute(stmt_select)
    print(resp)

    # Cleaning up, dropping the table again
    yield from cursor.execute(stmt_drop)
    yield from cursor.close()
    conn.close()


loop.run_until_complete(test_example_transaction())



import asyncio
import aiomysql


async def test_example(loop):
    conn = await aiomysql.connect(host='127.0.0.1', port=3306,
                                  user='root', password='',
                                  db='test_pymysql', loop=loop)

    async with conn.cursor() as cur:
        await cur.execute('DROP PROCEDURE IF EXISTS myinc;')
        await cur.execute("""CREATE PROCEDURE myinc(p1 INT)
                             BEGIN
                                 SELECT p1 + 1;
                             END""")

        await cur.callproc('myinc', [1])
        (ret, ) = await cur.fetchone()
        assert 2, ret
        print(ret)

    conn.close()


loop = asyncio.get_event_loop()
loop.run_until_complete(test_example(loop))



import asyncio

import pytest
from pymysql.err import ProgrammingError


@pytest.mark.run_loop
async def test_nextset(cursor):
    await cursor.execute("SELECT 1; SELECT 2;")
    r = await cursor.fetchall()
    assert [(1,)] == list(r)

    r = await cursor.nextset()
    assert r

    r = await cursor.fetchall()
    assert [(2,)] == list(r)
    res = await cursor.nextset()
    assert res is None


@pytest.mark.run_loop
async def test_skip_nextset(cursor):
    await cursor.execute("SELECT 1; SELECT 2;")
    r = await cursor.fetchall()
    assert [(1,)] == list(r)

    await cursor.execute("SELECT 42")
    r = await cursor.fetchall()
    assert [(42,)] == list(r)


@pytest.mark.run_loop
async def test_nextset_error(cursor):
    await cursor.execute("SELECT 1; xyzzy;")

    # nextset shouldn't hang on error, it should raise syntax error
    with pytest.raises(ProgrammingError):
        await asyncio.wait_for(cursor.nextset(), 5)


@pytest.mark.run_loop
async def test_ok_and_next(cursor):
    await cursor.execute("SELECT 1; commit; SELECT 2;")
    r = await cursor.fetchall()
    assert [(1,)] == list(r)

    res = await cursor.nextset()
    assert res

    res = await cursor.nextset()
    assert res

    r = await cursor.fetchall()
    assert [(2,)] == list(r)

    res = await cursor.nextset()
    assert res is None


@pytest.mark.xfail
@pytest.mark.run_loop
async def test_multi_cursorxx(connection):
    cur1 = await connection.cursor()
    cur2 = await connection.cursor()

    await cur1.execute("SELECT 1; SELECT 2;")
    await cur2.execute("SELECT 42")

    r1 = await cur1.fetchall()
    r2 = await cur2.fetchall()

    assert [(1,)] == list(r1)
    assert [(42,)] == list(r2)

    res = await cur1.nextset()
    assert res

    assert [(2,)] == list(r1)
    res = await cur1.nextset()
    assert res is None

    # TODO: How about SSCursor and nextset?
    # It's very hard to implement correctly...



from unittest import mock

import pytest
from sqlalchemy import MetaData, Table, Column, Integer, String
from sqlalchemy.schema import DropTable, CreateTable
from sqlalchemy.sql.expression import bindparam

import aiomysql
from aiomysql import sa, Cursor

meta = MetaData()
tbl = Table('sa_tbl', meta,
            Column('id', Integer, nullable=False,
                   primary_key=True),
            Column('name', String(255)))


@pytest.fixture()
def sa_connect(connection_creator):
    async def connect(**kwargs):
        conn = await connection_creator(**kwargs)
        await conn.autocommit(True)
        cur = await conn.cursor()
        await cur.execute("DROP TABLE IF EXISTS sa_tbl")
        await cur.execute("CREATE TABLE sa_tbl "
                          "(id serial, name varchar(255))")
        await cur.execute("INSERT INTO sa_tbl (name)"
                          "VALUES ('first')")

        await cur._connection.commit()
        # yield from cur.close()
        engine = mock.Mock()
        engine.dialect = sa.engine._dialect
        return sa.SAConnection(conn, engine)
    return connect


@pytest.mark.run_loop
async def test_execute_text_select(sa_connect):
    conn = await sa_connect()
    res = await conn.execute("SELECT * FROM sa_tbl;")
    assert isinstance(res.cursor, Cursor)
    assert ('id', 'name') == res.keys()
    rows = await res.fetchall()
    assert res.closed
    assert res.cursor is None
    assert 1 == len(rows)
    row = rows[0]
    assert 1 == row[0]
    assert 1 == row['id']
    assert 1 == row.id
    assert 'first' == row[1]
    assert 'first' == row['name']
    assert 'first' == row.name
    # TODO: fix this
    await conn._connection.commit()


@pytest.mark.run_loop
async def test_execute_sa_select(sa_connect):
    conn = await sa_connect()
    res = await conn.execute(tbl.select())
    assert isinstance(res.cursor, Cursor)
    assert ('id', 'name') == res.keys()
    rows = await res.fetchall()
    assert res.closed
    assert res.cursor is None
    assert res.returns_rows

    assert 1 == len(rows)
    row = rows[0]
    assert 1 == row[0]
    assert 1 == row['id']
    assert 1 == row.id
    assert 'first' == row[1]
    assert 'first' == row['name']
    assert 'first' == row.name
    # TODO: fix this
    await conn._connection.commit()


@pytest.mark.run_loop
async def test_execute_sa_insert_with_dict(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert(), {"id": 2, "name": "second"})

    res = await conn.execute(tbl.select())
    rows = await res.fetchall()
    assert 2 == len(rows)
    assert (1, 'first') == rows[0]
    assert (2, 'second') == rows[1]


@pytest.mark.run_loop
async def test_execute_sa_insert_with_tuple(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert(), (2, "second"))

    res = await conn.execute(tbl.select())
    rows = await res.fetchall()
    assert 2 == len(rows)
    assert (1, 'first') == rows[0]
    assert (2, 'second') == rows[1]


@pytest.mark.run_loop
async def test_execute_sa_insert_named_params(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert(), id=2, name="second")

    res = await conn.execute(tbl.select())
    rows = await res.fetchall()
    assert 2 == len(rows)
    assert (1, 'first') == rows[0]
    assert (2, 'second') == rows[1]


@pytest.mark.run_loop
async def test_execute_sa_insert_positional_params(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert(), 2, "second")

    res = await conn.execute(tbl.select())
    rows = await res.fetchall()
    assert 2 == len(rows)
    assert (1, 'first') == rows[0]
    assert (2, 'second') == rows[1]


@pytest.mark.run_loop
async def test_scalar(sa_connect):
    conn = await sa_connect()
    res = await conn.scalar(tbl.count())
    assert 1 == res


@pytest.mark.run_loop
async def test_scalar_None(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.delete())
    res = await conn.scalar(tbl.select())
    assert res is None
    # TODO: fix this
    await conn._connection.commit()


@pytest.mark.run_loop
async def test_row_proxy(sa_connect):
    conn = await sa_connect()
    res = await conn.execute(tbl.select())
    rows = await res.fetchall()
    row = rows[0]
    row2 = await (await conn.execute(tbl.select())).first()
    assert 2 == len(row)
    assert ['id', 'name'] == list(row)
    assert 'id' in row
    assert 'unknown' not in row
    assert 'first' == row.name
    assert 'first' == row[tbl.c.name]
    with pytest.raises(AttributeError):
        row.unknown
    assert "(1, 'first')" == repr(row)
    assert (1, 'first') == row.as_tuple()
    assert (555, 'other') != row.as_tuple()
    assert row2 == row
    assert 5 != row
    # TODO: fix this
    await conn._connection.commit()


@pytest.mark.run_loop
async def test_insert(sa_connect):
    conn = await sa_connect()
    res = await conn.execute(tbl.insert().values(name='second'))
    assert 1 == res.rowcount
    assert 2 == res.lastrowid


@pytest.mark.run_loop
async def test_raw_insert(sa_connect):
    conn = await sa_connect()
    await conn.execute(
        "INSERT INTO sa_tbl (name) VALUES ('third')")
    res = await conn.execute(tbl.select())
    assert 2 == res.rowcount
    assert ('id', 'name') == res.keys()
    assert res.returns_rows

    rows = await res.fetchall()
    assert 2 == len(rows)
    assert 2 == rows[1].id


@pytest.mark.run_loop
async def test_raw_insert_with_params(sa_connect):
    conn = await sa_connect()
    res = await conn.execute(
        "INSERT INTO sa_tbl (id, name) VALUES (%s, %s)",
        2, 'third')
    res = await conn.execute(tbl.select())
    assert 2 == res.rowcount
    assert ('id', 'name') == res.keys()
    assert res.returns_rows

    rows = await res.fetchall()
    assert 2 == len(rows)
    assert 2 == rows[1].id


@pytest.mark.run_loop
async def test_raw_insert_with_params_dict(sa_connect):
    conn = await sa_connect()
    res = await conn.execute(
        "INSERT INTO sa_tbl (id, name) VALUES (%(id)s, %(name)s)",
        {'id': 2, 'name': 'third'})
    res = await conn.execute(tbl.select())
    assert 2 == res.rowcount
    assert ('id', 'name') == res.keys()
    assert res.returns_rows

    rows = await res.fetchall()
    assert 2 == len(rows)
    assert 2 == rows[1].id


@pytest.mark.run_loop
async def test_raw_insert_with_named_params(sa_connect):
    conn = await sa_connect()
    res = await conn.execute(
        "INSERT INTO sa_tbl (id, name) VALUES (%(id)s, %(name)s)",
        id=2, name='third')
    res = await conn.execute(tbl.select())
    assert 2 == res.rowcount
    assert ('id', 'name') == res.keys()
    assert res.returns_rows

    rows = await res.fetchall()
    assert 2 == len(rows)
    assert 2 == rows[1].id


@pytest.mark.run_loop
async def test_raw_insert_with_executemany(sa_connect):
    conn = await sa_connect()
    # with pytest.raises(sa.ArgumentError):
    await conn.execute(
        "INSERT INTO sa_tbl (id, name) VALUES (%(id)s, %(name)s)",
        [{"id": 2, "name": 'third'}, {"id": 3, "name": 'forth'}])
    await conn.execute(
        tbl.update().where(
            tbl.c.id == bindparam("id")
        ).values(
            {"name": bindparam("name")}
        ),
        [
            {"id": 2, "name": "t2"},
            {"id": 3, "name": "t3"}
        ]
    )
    with pytest.raises(sa.ArgumentError):
        await conn.execute(
            DropTable(tbl),
            [{}, {}]
        )
    with pytest.raises(sa.ArgumentError):
        await conn.execute(
            {},
            [{}, {}]
        )


@pytest.mark.run_loop
async def test_raw_select_with_wildcard(sa_connect):
    conn = await sa_connect()
    await conn.execute(
        'SELECT * FROM sa_tbl WHERE name LIKE "%test%"')


@pytest.mark.run_loop
async def test_delete(sa_connect):
    conn = await sa_connect()

    res = await conn.execute(tbl.delete().where(tbl.c.id == 1))

    assert () == res.keys()
    assert 1 == res.rowcount
    assert not res.returns_rows
    assert res.closed
    assert res.cursor is None


@pytest.mark.run_loop
async def test_double_close(sa_connect):
    conn = await sa_connect()
    res = await conn.execute("SELECT 1")
    await res.close()
    assert res.closed
    assert res.cursor is None
    await res.close()
    assert res.closed
    assert res.cursor is None


@pytest.mark.run_loop
@pytest.mark.skip("Find out how to close cursor on __del__ method")
async def test_weakrefs(sa_connect):
    conn = await sa_connect()
    assert 0 == len(conn._weak_results)
    res = await conn.execute("SELECT 1")
    assert 1 == len(conn._weak_results)
    cur = res.cursor
    assert not cur.closed
    # TODO: fix this, how close cursor if result was deleted
    # yield from cur.close()
    del res
    assert cur.closed
    assert 0 == len(conn._weak_results)


@pytest.mark.run_loop
async def test_fetchall(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert().values(name='second'))

    res = await conn.execute(tbl.select())
    rows = await res.fetchall()
    assert 2 == len(rows)
    assert res.closed
    assert res.returns_rows
    assert [(1, 'first'), (2, 'second')] == rows


@pytest.mark.run_loop
async def test_fetchall_closed(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert().values(name='second'))

    res = await conn.execute(tbl.select())
    await res.close()
    with pytest.raises(sa.ResourceClosedError):
        await res.fetchall()


@pytest.mark.run_loop
async def test_fetchall_not_returns_rows(sa_connect):
    conn = await sa_connect()
    res = await conn.execute(tbl.delete())
    with pytest.raises(sa.ResourceClosedError):
        await res.fetchall()


@pytest.mark.run_loop
async def test_fetchone_closed(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert().values(name='second'))

    res = await conn.execute(tbl.select())
    await res.close()
    with pytest.raises(sa.ResourceClosedError):
        await res.fetchone()


@pytest.mark.run_loop
async def test_first_not_returns_rows(sa_connect):
    conn = await sa_connect()
    res = await conn.execute(tbl.delete())
    with pytest.raises(sa.ResourceClosedError):
        await res.first()


@pytest.mark.run_loop
async def test_fetchmany(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert().values(name='second'))

    res = await conn.execute(tbl.select())
    rows = await res.fetchmany()
    assert 1 == len(rows)
    assert not res.closed
    assert res.returns_rows
    assert [(1, 'first')] == rows


@pytest.mark.run_loop
async def test_fetchmany_with_size(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert().values(name='second'))

    res = await conn.execute(tbl.select())
    rows = await res.fetchmany(100)
    assert 2 == len(rows)
    assert not res.closed
    assert res.returns_rows
    assert [(1, 'first'), (2, 'second')] == rows


@pytest.mark.run_loop
async def test_fetchmany_closed(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert().values(name='second'))

    res = await conn.execute(tbl.select())
    await res.close()
    with pytest.raises(sa.ResourceClosedError):
        await res.fetchmany()


@pytest.mark.run_loop
async def test_fetchmany_with_size_closed(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert().values(name='second'))

    res = await conn.execute(tbl.select())
    await res.close()
    with pytest.raises(sa.ResourceClosedError):
        await res.fetchmany(5555)


@pytest.mark.run_loop
async def test_fetchmany_not_returns_rows(sa_connect):
    conn = await sa_connect()
    res = await conn.execute(tbl.delete())
    with pytest.raises(sa.ResourceClosedError):
        await res.fetchmany()


@pytest.mark.run_loop
async def test_fetchmany_close_after_last_read(sa_connect):
    conn = await sa_connect()

    res = await conn.execute(tbl.select())
    rows = await res.fetchmany()
    assert 1 == len(rows)
    assert not res.closed
    assert res.returns_rows
    assert [(1, 'first')] == rows
    rows2 = await res.fetchmany()
    assert 0 == len(rows2)
    assert res.closed


@pytest.mark.run_loop
async def test_create_table(sa_connect):
    conn = await sa_connect()
    res = await conn.execute(DropTable(tbl))
    with pytest.raises(sa.ResourceClosedError):
        await res.fetchmany()

    with pytest.raises(aiomysql.ProgrammingError):
        await conn.execute("SELECT * FROM sa_tbl")

    res = await conn.execute(CreateTable(tbl))
    with pytest.raises(sa.ResourceClosedError):
        await res.fetchmany()

    res = await conn.execute("SELECT * FROM sa_tbl")
    assert 0 == len(await res.fetchall())


@pytest.mark.run_loop
async def test_async_iter(sa_connect):
    conn = await sa_connect()
    await conn.execute(tbl.insert().values(name="second"))

    ret = []
    async for row in conn.execute(tbl.select()):
        ret.append(row)
    assert [(1, "first"), (2, "second")] == ret



import datetime

import pytest
from pymysql.err import Warning

import aiomysql


@pytest.mark.run_loop
async def test_issue_3(connection):
    """ undefined methods datetime_or_None, date_or_None """
    conn = connection
    c = await conn.cursor()
    await c.execute("drop table if exists issue3")
    await c.execute(
        "create table issue3 (d date, t time, dt datetime, ts timestamp)")
    try:
        await c.execute(
            "insert into issue3 (d, t, dt, ts) values (%s,%s,%s,%s)",
            (None, None, None, None))
        await c.execute("select d from issue3")
        r = await c.fetchone()
        assert r[0] is None
        await c.execute("select t from issue3")
        r = await c.fetchone()
        assert r[0] is None
        await c.execute("select dt from issue3")
        r = await c.fetchone()
        assert r[0] is None
        await c.execute("select ts from issue3")
        r = await c.fetchone()
        assert type(r[0]) in (type(None), datetime.datetime)
    finally:
        await c.execute("drop table issue3")


@pytest.mark.run_loop
async def test_issue_4(connection):
    """ can't retrieve TIMESTAMP fields """
    conn = connection
    c = await conn.cursor()
    await c.execute("drop table if exists issue4")
    await c.execute("create table issue4 (ts timestamp)")
    try:
        await c.execute("insert into issue4 (ts) values (now())")
        await c.execute("select ts from issue4")
        r = await c.fetchone()
        assert isinstance(r[0], datetime.datetime)
    finally:
        await c.execute("drop table issue4")


@pytest.mark.run_loop
async def test_issue_5(connection):
    """ query on information_schema.tables fails """
    conn = connection
    cur = await conn.cursor()
    await cur.execute("select * from information_schema.tables")


@pytest.mark.run_loop
async def test_issue_6(connection_creator):
    # test for exception: TypeError: ord() expected a character,
    # but string of length 0 found
    conn = await connection_creator(db='mysql')
    c = await conn.cursor()
    assert conn.db == 'mysql'
    await c.execute("select * from user")
    await conn.ensure_closed()


@pytest.mark.run_loop
async def test_issue_8(connection):
    """ Primary Key and Index error when selecting data """
    conn = connection
    c = await conn.cursor()
    await c.execute("drop table if exists test")
    await c.execute("""CREATE TABLE `test` (
        `station` int(10) NOT NULL DEFAULT '0',
        `dh` datetime NOT NULL DEFAULT '2020-04-25 22:39:12',
        `echeance` int(1) NOT NULL DEFAULT '0', `me` double DEFAULT NULL,
        `mo` double DEFAULT NULL, PRIMARY
        KEY (`station`,`dh`,`echeance`)) ENGINE=MyISAM DEFAULT
        CHARSET=latin1;""")
    try:
        await c.execute("SELECT * FROM test")
        assert 0 == c.rowcount
        await c.execute(
            "ALTER TABLE `test` ADD INDEX `idx_station` (`station`)")
        await c.execute("SELECT * FROM test")
        assert 0 == c.rowcount
    finally:
        await c.execute("drop table test")


@pytest.mark.run_loop
async def test_issue_13(connection):
    """ can't handle large result fields """
    conn = connection
    cur = await conn.cursor()
    await cur.execute("drop table if exists issue13")
    try:
        await cur.execute("create table issue13 (t text)")
        # ticket says 18k
        size = 18 * 1024
        await cur.execute("insert into issue13 (t) values (%s)",
                          ("x" * size,))
        await cur.execute("select t from issue13")
        # use assertTrue so that obscenely huge error messages don't print
        r = await cur.fetchone()
        assert "x" * size == r[0]
    finally:
        await cur.execute("drop table issue13")


@pytest.mark.run_loop
async def test_issue_15(connection):
    """ query should be expanded before perform character encoding """
    conn = connection
    c = await conn.cursor()
    await c.execute("drop table if exists issue15")
    await c.execute("create table issue15 (t varchar(32))")
    try:
        await c.execute("insert into issue15 (t) values (%s)",
                        (u'\xe4\xf6\xfc',))
        await c.execute("select t from issue15")
        r = await c.fetchone()
        assert u'\xe4\xf6\xfc' == r[0]
    finally:
        await c.execute("drop table issue15")


@pytest.mark.run_loop
async def test_issue_16(connection):
    """ Patch for string and tuple escaping """
    conn = connection
    c = await conn.cursor()
    await c.execute("drop table if exists issue16")
    await c.execute("create table issue16 (name varchar(32) "
                    "primary key, email varchar(32))")
    try:
        await c.execute("insert into issue16 (name, email) values "
                        "('pete', 'floydophone')")
        await c.execute("select email from issue16 where name=%s",
                        ("pete",))
        r = await c.fetchone()
        assert "floydophone" == r[0]
    finally:
        await c.execute("drop table issue16")


@pytest.mark.skip(
    "test_issue_17() requires a custom, legacy MySQL configuration and "
    "will not be run.")
@pytest.mark.run_loop
async def test_issue_17(connection, connection_creator, mysql_params):
    """ could not connect mysql use passwod """
    conn = connection
    c = await conn.cursor()
    db = mysql_params['db']
    # grant access to a table to a user with a password
    try:
        await c.execute("drop table if exists issue17")
        await c.execute(
            "create table issue17 (x varchar(32) primary key)")
        await c.execute(
            "insert into issue17 (x) values ('hello, world!')")
        await c.execute("grant all privileges on %s.issue17 to "
                        "'issue17user'@'%%' identified by '1234'"
                        % db)
        await conn.commit()

        conn2 = await connection_creator(user="issue17user",
                                         passwd="1234")
        c2 = await conn2.cursor()
        await c2.execute("select x from issue17")
        r = await c2.fetchone()
        assert "hello == world!", r[0]
    finally:
        await c.execute("drop table issue17")


@pytest.mark.run_loop
async def test_issue_34(connection_creator):
    try:
        await connection_creator(host="localhost", port=1237,
                                 user="root", unix_socket=None)
        pytest.fail()
    except aiomysql.OperationalError as e:
        assert 2003 == e.args[0]
    except Exception:
        pytest.fail()


@pytest.mark.run_loop
async def test_issue_33(connection_creator):
    conn = await connection_creator(charset='utf8')
    c = await conn.cursor()
    try:
        await c.execute(
            b"drop table if exists hei\xc3\x9fe".decode("utf8"))
        await c.execute(
            b"create table hei\xc3\x9fe (name varchar(32))".decode("utf8"))
        await c.execute(b"insert into hei\xc3\x9fe (name) "
                        b"values ('Pi\xc3\xb1ata')".
                        decode("utf8"))
        await c.execute(
            b"select name from hei\xc3\x9fe".decode("utf8"))
        r = await c.fetchone()
        assert b"Pi\xc3\xb1ata".decode("utf8") == r[0]
    finally:
        await c.execute(b"drop table hei\xc3\x9fe".decode("utf8"))


@pytest.mark.skip("This test requires manual intervention")
@pytest.mark.run_loop
async def test_issue_35(connection):
    conn = connection
    c = await conn.cursor()
    print("sudo killall -9 mysqld within the next 10 seconds")
    try:
        await c.execute("select sleep(10)")
        pytest.fail()
    except aiomysql.OperationalError as e:
        assert 2013 == e.args[0]


@pytest.mark.run_loop
async def test_issue_36(connection_creator):
    conn = await connection_creator()
    c = await conn.cursor()
    # kill connections[0]
    await c.execute("show processlist")
    kill_id = None
    rows = await c.fetchall()
    for row in rows:
        id = row[0]
        info = row[7]
        if info == "show processlist":
            kill_id = id
            break
    try:
        # now nuke the connection
        await conn.kill(kill_id)
        # make sure this connection has broken
        await c.execute("show tables")
        pytest.fail()
    except Exception:
        pass

    # check the process list from the other connection
    conn2 = await connection_creator()
    c = await conn2.cursor()
    await c.execute("show processlist")
    rows = await c.fetchall()
    ids = [row[0] for row in rows]

    assert kill_id not in ids


@pytest.mark.run_loop
async def test_issue_37(connection):
    conn = connection
    c = await conn.cursor()
    assert 1 == (await c.execute("SELECT @foo"))

    r = await c.fetchone()
    assert (None,) == r
    assert 0 == (await c.execute("SET @foo = 'bar'"))
    await c.execute("set @foo = 'bar'")


@pytest.mark.run_loop
async def test_issue_38(connection):
    conn = connection
    c = await conn.cursor()
    # reduced size for most default mysql installs
    datum = "a" * 1024 * 1023

    try:
        await c.execute("drop table if exists issue38")
        await c.execute(
            "create table issue38 (id integer, data mediumblob)")
        await c.execute("insert into issue38 values (1, %s)",
                        (datum,))
    finally:
        await c.execute("drop table issue38")


@pytest.mark.run_loop
async def disabled_test_issue_54(connection):
    conn = connection
    c = await conn.cursor()
    await c.execute("drop table if exists issue54")
    big_sql = "select * from issue54 where "
    big_sql += " and ".join("%d=%d" % (i, i) for i in range(0, 100000))

    try:
        await c.execute(
            "create table issue54 (id integer primary key)")
        await c.execute("insert into issue54 (id) values (7)")
        await c.execute(big_sql)

        r = await c.fetchone()
        assert 7 == r[0]
    finally:
        await c.execute("drop table issue54")


@pytest.mark.run_loop
async def test_issue_66(connection):
    """ 'Connection' object has no attribute 'insert_id' """
    conn = connection
    c = await conn.cursor()
    assert 0 == conn.insert_id()
    try:
        await c.execute("drop table if exists issue66")
        await c.execute("create table issue66 (id integer primary "
                        "key auto_increment, x integer)")
        await c.execute("insert into issue66 (x) values (1)")
        await c.execute("insert into issue66 (x) values (1)")
        assert 2 == conn.insert_id()
    finally:
        await c.execute("drop table issue66")


@pytest.mark.run_loop
async def test_issue_79(connection):
    """ Duplicate field overwrites the previous one in the result
    of DictCursor """
    conn = connection
    c = await conn.cursor(aiomysql.cursors.DictCursor)

    await c.execute("drop table if exists a")
    await c.execute("drop table if exists b")
    await c.execute("""CREATE TABLE a (id int, value int)""")
    await c.execute("""CREATE TABLE b (id int, value int)""")

    a = (1, 11)
    b = (1, 22)
    try:
        await c.execute("insert into a values (%s, %s)", a)
        await c.execute("insert into b values (%s, %s)", b)

        await c.execute("SELECT * FROM a inner join b on a.id = b.id")
        r, *_ = await c.fetchall()
        assert r['id'] == 1
        assert r['value'] == 11
        assert r['b.value'] == 22
    finally:
        await c.execute("drop table a")
        await c.execute("drop table b")


@pytest.mark.run_loop
async def test_issue_95(connection):
    """ Leftover trailing OK packet for "CALL my_sp" queries """
    conn = connection
    cur = await conn.cursor()
    await cur.execute("DROP PROCEDURE IF EXISTS `foo`")
    await cur.execute("""CREATE PROCEDURE `foo` ()
    BEGIN
        SELECT 1;
    END""")
    try:
        await cur.execute("""CALL foo()""")
        await cur.execute("""SELECT 1""")
        r = await cur.fetchone()
        assert r[0] == 1
    finally:
        await cur.execute("DROP PROCEDURE IF EXISTS `foo`")


@pytest.mark.run_loop
async def test_issue_114(connection_creator):
    """ autocommit is not set after reconnecting with ping() """
    conn = await connection_creator(charset="utf8")
    await conn.autocommit(False)
    c = await conn.cursor()
    await c.execute("""select @@autocommit;""")
    r = await c.fetchone()
    assert not r[0]
    await conn.ensure_closed()
    await conn.ping()
    await c.execute("""select @@autocommit;""")
    r = await c.fetchone()
    assert not r[0]
    await conn.ensure_closed()

    # Ensure autocommit() is still working
    conn = await connection_creator(charset="utf8")
    c = await conn.cursor()
    await c.execute("""select @@autocommit;""")
    r = await c.fetchone()
    assert not r[0]
    await conn.ensure_closed()
    await conn.ping()
    await conn.autocommit(True)
    await c.execute("""select @@autocommit;""")
    r = await c.fetchone()
    assert r[0]
    await conn.ensure_closed()


@pytest.mark.run_loop
async def test_issue_175(connection):
    """ The number of fields returned by server is read in wrong way """
    conn = connection
    cur = await conn.cursor()
    for length in (200, 300):
        cols = ', '.join('c{0} integer'.format(i) for i in range(length))
        sql = 'create table test_field_count ({0})'.format(cols)
        try:
            await cur.execute(sql)
            await cur.execute('select * from test_field_count')
            assert len(cur.description) == length
        finally:
            await cur.execute('drop table if exists test_field_count')


# MySQL will get you to renegotiate if sent a cleartext password
@pytest.mark.run_loop
async def test_issue_323(mysql_server, loop, recwarn):
    async with aiomysql.create_pool(**mysql_server['conn_params'],
                                    loop=loop) as pool:
        async with pool.get() as conn:
            async with conn.cursor() as cur:
                drop_db = "DROP DATABASE IF EXISTS bugtest;"
                await cur.execute(drop_db)

                create_db = "CREATE DATABASE bugtest;"
                await cur.execute(create_db)

                create_table = """CREATE TABLE IF NOT EXISTS `bugtest`.`testtable` (
                `id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
                `bindata` VARBINARY(200) NOT NULL,
                PRIMARY KEY (`id`)
                );"""

                await cur.execute(create_table)

            try:
                recwarn.clear()

                async with conn.cursor() as cur:
                    await cur.execute("INSERT INTO `bugtest`.`testtable` "
                                      "(bindata) VALUES (%s);",
                                      (b'\xB0\x17',))

                    warnings = [warn for warn in recwarn.list
                                if warn.category is Warning]
                    assert len(warnings) == 0, \
                        "Got unexpected MySQL warning {}".\
                        format(' '.join(str(x) for x in warnings))

                    await cur.execute("SELECT * FROM `bugtest`.`testtable`;")
                    rows = await cur.fetchall()

                    assert len(rows) == 1, "Table should have 1 row"

            finally:
                async with conn.cursor() as cur:
                    await cur.execute("DELETE FROM `bugtest`.`testtable`;")



import copy
from aiomysql import create_pool

import pytest


# You could parameterise these tests with this, but then pytest
# does some funky stuff and spins up and tears down containers
# per function call.  Remember it would be
# mysql_versions * event_loops * 4 auth tests ~= 3*2*4 ~= 24 tests

# As the MySQL daemon restarts at least 3 times in the container
# before it becomes stable, there's a sleep(10) so that's
# around a 4min wait time.

# @pytest.mark.parametrize("user,password,plugin", [
#     ("nopass_sha256", None, 'sha256_password'),
#     ("user_sha256", 'pass_sha256', 'sha256_password'),
#     ("nopass_caching_sha2", None, 'caching_sha2_password'),
#     ("user_caching_sha2", 'pass_caching_sha2', 'caching_sha2_password'),
# ])


def ensure_mysql_version(mysql_server):
    if mysql_server["db_type"] != "mysql" \
            or mysql_server["server_version_tuple_short"] != (8, 0):
        pytest.skip("Not applicable for {0} version: {1}"
                    .format(mysql_server["db_type"],
                            mysql_server["server_version_tuple_short"]))


@pytest.mark.run_loop
async def test_sha256_nopw(mysql_server, loop):
    ensure_mysql_version(mysql_server)

    connection_data = copy.copy(mysql_server['conn_params'])
    connection_data['user'] = 'nopass_sha256'
    connection_data['password'] = None

    async with create_pool(**connection_data,
                           loop=loop) as pool:
        async with pool.get() as conn:
            # User doesnt have any permissions to look at DBs
            # But as 8.0 will default to caching_sha2_password
            assert conn._auth_plugin_used == 'sha256_password'


@pytest.mark.run_loop
async def test_sha256_pw(mysql_server, loop):
    ensure_mysql_version(mysql_server)

    # https://dev.mysql.com/doc/refman/8.0/en/sha256-pluggable-authentication.html
    # Unlike caching_sha2_password, the sha256_password plugin does not treat
    # shared-memory connections as secure, even though share-memory transport
    # is secure by default.
    if "unix_socket" in mysql_server['conn_params']:
        pytest.skip("sha256_password is not supported on unix sockets")

    connection_data = copy.copy(mysql_server['conn_params'])
    connection_data['user'] = 'user_sha256'
    connection_data['password'] = 'pass_sha256'

    async with create_pool(**connection_data,
                           loop=loop) as pool:
        async with pool.get() as conn:
            # User doesnt have any permissions to look at DBs
            # But as 8.0 will default to caching_sha2_password
            assert conn._auth_plugin_used == 'sha256_password'


@pytest.mark.run_loop
async def test_cached_sha256_nopw(mysql_server, loop):
    ensure_mysql_version(mysql_server)

    connection_data = copy.copy(mysql_server['conn_params'])
    connection_data['user'] = 'nopass_caching_sha2'
    connection_data['password'] = None

    async with create_pool(**connection_data,
                           loop=loop) as pool:
        async with pool.get() as conn:
            # User doesnt have any permissions to look at DBs
            # But as 8.0 will default to caching_sha2_password
            assert conn._auth_plugin_used == 'caching_sha2_password'


@pytest.mark.run_loop
async def test_cached_sha256_pw(mysql_server, loop):
    ensure_mysql_version(mysql_server)

    connection_data = copy.copy(mysql_server['conn_params'])
    connection_data['user'] = 'user_caching_sha2'
    connection_data['password'] = 'pass_caching_sha2'

    async with create_pool(**connection_data,
                           loop=loop) as pool:
        async with pool.get() as conn:
            # User doesnt have any permissions to look at DBs
            # But as 8.0 will default to caching_sha2_password
            assert conn._auth_plugin_used == 'caching_sha2_password'
